@@ -31,10 +31,25 @@ from gi.repository import GLib, GObject, Gio, Gdk, Gtk, Pango, Adw
 from .cmb_tutor import CmbTutor, CmbTutorState
 from . import cmb_tutorial
 
-from cambalache import CmbProject, CmbUI, CmbCSS, CmbObject, CmbTypeChooserPopover, getLogger, config, utils, _
+from cambalache import (
+    CmbProject,
+    CmbUI,
+    CmbCSS,
+    CmbObject,
+    CmbGResource,
+    CmbGResourceEditor,
+    CmbTypeChooserPopover,
+    getLogger,
+    notification_center,
+    config,
+    utils,
+    _
+)
 
 logger = getLogger(__name__)
 
+GObject.type_ensure(CmbGResourceEditor.__gtype__)
+
 
 @Gtk.Template(resource_path="/ar/xjuan/Cambalache/app/cmb_window.ui")
 class CmbWindow(Adw.ApplicationWindow):
@@ -46,8 +61,12 @@ class CmbWindow(Adw.ApplicationWindow):
     }
 
     open_filter = Gtk.Template.Child()
-    gtk3_import_filter = Gtk.Template.Child()
-    gtk4_import_filter = Gtk.Template.Child()
+    gtk4_filter = Gtk.Template.Child()
+    gtk3_filter = Gtk.Template.Child()
+    gtk_builder_filter = Gtk.Template.Child()
+    glade_filter = Gtk.Template.Child()
+    css_filter = Gtk.Template.Child()
+    gresource_filter = Gtk.Template.Child()
 
     headerbar = Gtk.Template.Child()
     title = Gtk.Template.Child()
@@ -59,6 +78,9 @@ class CmbWindow(Adw.ApplicationWindow):
 
     # Start screen
     version_label = Gtk.Template.Child()
+    front_notification_list_view = Gtk.Template.Child()
+    notification_dialog = Gtk.Template.Child()
+    notification_list_view = Gtk.Template.Child()
 
     # New Project
     np_name_entry = Gtk.Template.Child()
@@ -73,8 +95,10 @@ class CmbWindow(Adw.ApplicationWindow):
     message_label = Gtk.Template.Child()
 
     # Workspace
+    workspace_stack = Gtk.Template.Child()
     view = Gtk.Template.Child()
-    column_view = Gtk.Template.Child()
+    source_view = Gtk.Template.Child()
+    list_view = Gtk.Template.Child()
     type_chooser = Gtk.Template.Child()
     editor_stack = Gtk.Template.Child()
     ui_editor = Gtk.Template.Child()
@@ -86,6 +110,7 @@ class CmbWindow(Adw.ApplicationWindow):
     accessible_editor = Gtk.Template.Child()
     signal_editor = Gtk.Template.Child()
     css_editor = Gtk.Template.Child()
+    gresource_editor = Gtk.Template.Child()
 
     # Tutor widgets
     intro_button = Gtk.Template.Child()
@@ -98,7 +123,7 @@ class CmbWindow(Adw.ApplicationWindow):
     FULLSCREEN = 1 << 4
 
     __portal_access_msg = _(
-        "Cambalache will not have access to the file for export because it is outside of your home directory. "
+        "Cambalache will not have access to the file for save because it is outside of your home directory. "
         "Consider using Flatseal or flatpak to give permissions to the host directory."
     )
 
@@ -109,6 +134,15 @@ class CmbWindow(Adw.ApplicationWindow):
 
         super().__init__(**kwargs)
 
+        self.gtk4_import_filters = Gio.ListStore()
+
+        for filter in [self.gtk4_filter, self.gtk_builder_filter, self.css_filter, self.gresource_filter]:
+            self.gtk4_import_filters.append(filter)
+
+        self.gtk3_import_filters = Gio.ListStore()
+        for filter in [self.gtk3_filter, self.gtk_builder_filter, self.glade_filter, self.css_filter, self.gresource_filter]:
+            self.gtk3_import_filters.append(filter)
+
         self.__recent_manager = self.__get_recent_manager()
 
         self.editor_stack.set_size_request(420, -1)
@@ -118,6 +152,7 @@ class CmbWindow(Adw.ApplicationWindow):
         for action in [
             "about",
             "add_css",
+            "add_gresource",
             "add_object",
             "add_object_toplevel",
             "add_placeholder",
@@ -132,12 +167,12 @@ class CmbWindow(Adw.ApplicationWindow):
             "debug",
             "delete",
             "donate",
-            "export",
             "import",
             "inspect",
             "intro",
             "liberapay",
             "new",
+            "notification",
             "open",
             "paste",
             "patreon",
@@ -178,7 +213,6 @@ class CmbWindow(Adw.ApplicationWindow):
             ("win.save", ["<Primary>s"]),
             ("win.save_as", ["<Shift><Primary>s"]),
             ("win.import", ["<Primary>i"]),
-            ("win.export", ["<Primary>e"]),
             ("win.close", ["<Primary>w"]),
             ("win.undo", ["<Primary>z"]),
             ("win.redo", ["<Primary><shift>z"]),
@@ -279,6 +313,10 @@ class CmbWindow(Adw.ApplicationWindow):
         self.__recent_manager.connect("changed", lambda rm: self.__update_recent_menu())
         self.__update_recent_menu()
 
+        self.notification_list_view.notification_center = notification_center
+        self.front_notification_list_view.notification_center = notification_center
+        notification_center.connect("new-notification", self.__on_new_notification)
+
     def __get_recent_manager(self):
         # Load the user host recently used file
         if os.environ.get("container", None) == "flatpak":
@@ -301,12 +339,13 @@ class CmbWindow(Adw.ApplicationWindow):
         if self.__project:
             self.__project.disconnect_by_func(self.__on_project_filename_notify)
             self.__project.disconnect_by_func(self.__on_project_selection_changed)
+            self.__project.disconnect_by_func(self.__on_project_gresource_changed)
             self.__project.disconnect_by_func(self.__on_project_changed)
 
         self.__project = project
         self.view.project = project
         self.type_chooser.project = project
-        self.column_view.project = project
+        self.list_view.project = project
 
         # Clear Editors
         self.ui_editor.object = None
@@ -321,6 +360,7 @@ class CmbWindow(Adw.ApplicationWindow):
         if project:
             self.__project.connect("notify::filename", self.__on_project_filename_notify)
             self.__project.connect("selection-changed", self.__on_project_selection_changed)
+            self.__project.connect("gresource-changed", self.__on_project_gresource_changed)
             self.__project.connect("changed", self.__on_project_changed)
             self.__on_project_selection_changed(project)
 
@@ -403,20 +443,6 @@ class CmbWindow(Adw.ApplicationWindow):
         )
         popover.popup()
 
-    @Gtk.Template.Callback("on_ui_editor_remove_ui")
-    def __on_ui_editor_remove_ui(self, editor, ui):
-        self.__remove_object_with_confirmation(ui)
-
-    @Gtk.Template.Callback("on_ui_editor_export_ui")
-    def __on_ui_editor_export_ui(self, editor, ui):
-        if ui.export():
-            self._show_message(_("File Exported"))
-
-    @Gtk.Template.Callback("on_css_editor_remove_ui")
-    def __on_css_editor_remove_ui(self, editor):
-        self.__remove_object_with_confirmation(editor.object)
-        return True
-
     def __on_focus_widget_notify(self, obj, pspec):
         widget = self.props.focus_widget
 
@@ -436,6 +462,7 @@ class CmbWindow(Adw.ApplicationWindow):
         sensitive = len(editable.get_chars(0, -1)) != 0
         self.np_location_chooser.set_sensitive(sensitive)
         self.np_ui_entry.set_sensitive(sensitive)
+        self.__update_action_new()
 
     def __update_dark_mode(self, style_manager):
         if style_manager.props.dark:
@@ -506,26 +533,55 @@ class CmbWindow(Adw.ApplicationWindow):
         self.__update_action_undo_redo()
         self.__update_action_save()
 
+    def __update_gresource_view(self):
+        if self.workspace_stack.get_visible_child_name() != "gresource":
+            return
+
+        sel = self.project.get_selection()
+        if len(sel) < 1:
+            return
+
+        obj = sel[0]
+
+        if isinstance(obj, CmbGResource):
+            gresource_id = obj.gresources_bundle.gresource_id
+            resource_xml = self.project.db.gresource_tostring(gresource_id)
+        else:
+            resource_xml = ""
+
+        self.source_view.buffer.set_text(resource_xml)
+
+    def __on_project_gresource_changed(self, project, gresource, field):
+        self.__update_gresource_view()
+
     def __on_project_selection_changed(self, project):
         sel = project.get_selection()
         self.__update_action_clipboard()
 
         obj = sel[0] if len(sel) > 0 else None
 
-        if type(obj) is CmbUI:
+        if isinstance(obj, CmbUI):
             self.ui_editor.object = obj
             self.ui_requires_editor.object = obj
             self.ui_fragment_editor.object = obj
+            self.workspace_stack.set_visible_child_name("ui")
             self.editor_stack.set_visible_child_name("ui")
             obj = None
-        elif type(obj) is CmbObject:
+        elif isinstance(obj, CmbObject):
+            self.workspace_stack.set_visible_child_name("ui")
             self.editor_stack.set_visible_child_name("object")
             if obj:
                 self.__user_message_by_type(obj.info)
-        elif type(obj) is CmbCSS:
+        elif isinstance(obj, CmbCSS):
             self.css_editor.object = obj
             self.editor_stack.set_visible_child_name("css")
             obj = None
+        elif isinstance(obj, CmbGResource):
+            self.gresource_editor.object = obj
+            self.workspace_stack.set_visible_child_name("gresource")
+            self.editor_stack.set_visible_child_name("gresource")
+            self.__update_gresource_view()
+            obj = None
 
         self.object_editor.object = obj
 
@@ -546,6 +602,9 @@ class CmbWindow(Adw.ApplicationWindow):
 
         self.intro_button.set_visible(enabled)
 
+    def __update_action_notification(self):
+        self.actions["notification"].set_enabled(len(notification_center.store) > 0)
+
     def __update_action_add_object(self):
         has_project = self.__is_project_visible()
         has_selection = True if self.project and len(self.project.get_selection()) > 0 else False
@@ -586,24 +645,39 @@ class CmbWindow(Adw.ApplicationWindow):
         else:
             self.title.remove_css_class("changed")
 
+    def __update_action_new(self):
+        self.actions["new"].set_enabled(len(self.np_name_entry.props.text) > 0)
+
     def __update_actions(self):
         has_project = self.__is_project_visible()
 
-        for action in ["save_as", "add_ui", "add_css", "delete", "import", "export", "close", "debug"]:
+        for action in [
+            "save_as",
+            "add_ui",
+            "add_css",
+            "add_gresource",
+            "delete",
+            "import",
+            "close",
+            "debug"
+        ]:
             self.actions[action].set_enabled(has_project)
 
+        self.__update_action_new()
         self.__update_action_save()
         self.__update_action_intro()
+        self.__update_action_notification()
         self.__update_action_clipboard()
         self.__update_action_undo_redo()
         self.__update_action_add_object()
         self.__update_action_remove_parent()
 
-    def __file_open_dialog_new(self, title, filter_obj=None, accept_label=None, use_project_dir=False):
+    def __file_open_dialog_new(self, title, filter_obj=None, accept_label=None, use_project_dir=False, filters=None):
         dialog = Gtk.FileDialog(
             modal=True,
             title=title,
             default_filter=filter_obj,
+            filters=filters,
             accept_label=accept_label,
         )
 
@@ -654,9 +728,13 @@ class CmbWindow(Adw.ApplicationWindow):
             basename = os.path.basename(filename)
             name, ext = os.path.splitext(basename)
 
-            if target_tk is None:
+            if not target_tk:
                 target_tk = CmbProject.get_target_from_ui_file(filename)
 
+            if not target_tk:
+                self.ask_gtk_version(filename)
+                return
+
             self.project = CmbProject(filename=os.path.join(dirname, f"{name}.cmb"), target_tk=target_tk)
             self.__set_page("workspace")
             self.__update_actions()
@@ -693,14 +771,14 @@ class CmbWindow(Adw.ApplicationWindow):
                     first_msg = _("Cambalache encounter the following issues:")
 
                     # Translators: this is the last message after the list of unsupported features
-                    last_msg = _("Your file will be exported as '{name}.cmb.ui' to avoid data loss.").format(name=name)
+                    last_msg = _("Your file will be saved as '{name}.cmb.ui' to avoid data loss.").format(name=name)
 
                     unsupported_features_list = [first_msg] + list + [last_msg]
                 else:
                     unsupported_feature = msg[0]
                     text = _(
                         "Cambalache encounter {unsupported_feature}\n"
-                        "Your file will be exported as '{name}.cmb.ui' to avoid data loss."
+                        "Your file will be saved as '{name}.cmb.ui' to avoid data loss."
                     ).format(unsupported_feature=unsupported_feature, name=name)
 
                 self.present_message_to_user(
@@ -864,7 +942,7 @@ class CmbWindow(Adw.ApplicationWindow):
             else:
                 self.project.redo()
         except Exception as e:
-            logger.warning(f"Undo/Redo error", exc_info=True)
+            logger.warning("Undo/Redo error", exc_info=True)
             self.present_message_to_user(
                 _("Undo/Redo stack got corrupted"),
                 secondary_text=_("Please try to reproduce and file an issue\n Error: {msg}").format(msg=str(e))
@@ -937,10 +1015,12 @@ class CmbWindow(Adw.ApplicationWindow):
 
         def on_dialog_response(dialog, response):
             if response == Gtk.ResponseType.YES:
-                if type(obj) is CmbUI:
+                if isinstance(obj, CmbUI):
                     self.project.remove_ui(obj)
-                elif type(obj) is CmbCSS:
+                elif isinstance(obj, CmbCSS):
                     self.project.remove_css(obj)
+                elif isinstance(obj, CmbGResource):
+                    self.project.remove_gresource(obj)
 
             dialog.destroy()
 
@@ -968,10 +1048,18 @@ class CmbWindow(Adw.ApplicationWindow):
 
         selection = self.project.get_selection()
         for obj in selection:
-            if type(obj) is CmbObject:
-                self.project.remove_object(obj)
-            else:
-                self.__remove_object_with_confirmation(obj)
+            try:
+                if isinstance(obj, CmbObject):
+                    self.project.remove_object(obj)
+                elif isinstance(obj, CmbGResource):
+                    if obj.resource_type == "gresources":
+                        self.__remove_object_with_confirmation(obj)
+                    else:
+                        self.project.remove_gresource(obj)
+                else:
+                    self.__remove_object_with_confirmation(obj)
+            except Exception as e:
+                self.present_message_to_user(_("Error deleting {name}").format(name=obj.display_name_type), secondary_text=str(e))
 
     def _on_add_object_activate(self, action, data):
         info = self.type_chooser.props.selected_type
@@ -1020,13 +1108,13 @@ class CmbWindow(Adw.ApplicationWindow):
             first_msg = _("Cambalache encounter the following issues:")
 
             # Translators: this is the last message after the list of unsupported features
-            last_msg = _("Your file will be exported as '{name}.cmb.ui' to avoid data loss.").format(name=name)
+            last_msg = _("Your file will be saved as '{name}.cmb.ui' to avoid data loss.").format(name=name)
 
             unsupported_features_list = [first_msg] + list + [last_msg]
         else:
             unsupported_feature = msg[0]
             text = _(
-                "Cambalache encounter {unsupported_feature}\nYour file will be exported as '{name}.cmb.ui' to avoid data loss."
+                "Cambalache encounter {unsupported_feature}\nYour file will be saved as '{name}.cmb.ui' to avoid data loss."
             ).format(unsupported_feature=unsupported_feature, name=name)
 
         self.present_message_to_user(
@@ -1042,22 +1130,39 @@ class CmbWindow(Adw.ApplicationWindow):
         def dialog_callback(dialog, res):
             try:
                 for file in dialog.open_multiple_finish(res):
-                    self.import_file(file.get_path())
+                    path = file.get_path()
+                    content_type = utils.content_type_guess(path)
+
+                    print("IMPORT", path, content_type)
+
+                    if content_type in ["application/x-gtk-builder", "application/x-glade"]:
+                        self.import_file(file.get_path())
+                    elif content_type == "text/css":
+                        self.project.add_css(path)
+                    elif content_type == "application/xml" and path.endswith("gresource.xml"):
+                        self.project.import_gresource(path)
             except Exception as e:
                 logger.warning(f"Error {e}")
 
         if self.project.target_tk == "gtk-4.0":
-            import_filter = self.gtk4_import_filter
+            import_filters = self.gtk4_import_filters
         else:
-            import_filter = self.gtk3_import_filter
+            import_filters = self.gtk3_import_filters
 
         dialog = self.__file_open_dialog_new(
             _("Choose file to import"),
-            filter_obj=import_filter,
+            filters=import_filters,
             accept_label=_("Import")
         )
         dialog.open_multiple(self, None, dialog_callback)
 
+    def _on_add_gresource_activate(self, action, data):
+        if self.project is None:
+            return
+
+        gresource = self.project.add_gresource("gresources")
+        self.project.set_selection([gresource])
+
     def __save(self):
         if self.project.save():
             self.__last_saved_index = self.project.history_index
@@ -1078,16 +1183,6 @@ class CmbWindow(Adw.ApplicationWindow):
 
         return False
 
-    def _on_export_activate(self, action, data):
-        if self.project is None:
-            return
-
-        self.save_project()
-
-        n = self.project.export()
-
-        self._show_message(_("{n} files exported").format(n=n) if n > 1 else _("File exported"))
-
     def _close_project_dialog_new(self):
         text = _("Save changes before closing?")
         dialog = Gtk.MessageDialog(
@@ -1187,20 +1282,19 @@ class CmbWindow(Adw.ApplicationWindow):
             about.props.translator_credits = "\n".join(translator_list)
 
     def _on_about_activate(self, action, data):
-        about = Adw.AboutWindow.new_from_appdata("/ar/xjuan/Cambalache/app/metainfo.xml", config.VERSION)
+        about = Adw.AboutDialog.new_from_appdata("/ar/xjuan/Cambalache/app/metainfo.xml", config.VERSION)
 
-        about.props.transient_for = self
         about.props.artists = [
             "Franco Dodorico",
             "Juan Pablo Ugarte",
         ]
-        about.props.copyright = "© 2020-2024 Juan Pablo Ugarte"
+        about.props.copyright = "© 2020-2025 Juan Pablo Ugarte"
         about.props.license_type = Gtk.License.LGPL_2_1_ONLY
 
         self.__update_translators(about)
         self.__populate_supporters(about)
 
-        about.present()
+        about.present(self)
 
     def _on_add_parent_activate(self, action, data):
         obj = self.project.get_selection()[0]
@@ -1295,7 +1389,7 @@ class CmbWindow(Adw.ApplicationWindow):
         elif node in ["add-ui", "add-window", "add-grid", "add-button"]:
             self.tutor_waiting_for_user_action = True
             self.tutor.pause()
-        elif node in ["donate", "export_all"]:
+        elif node in ["donate"]:
             self.menu_button.popdown()
         elif node == "show-type-popover":
             widget.props.popover.popdown()
@@ -1424,4 +1518,13 @@ class CmbWindow(Adw.ApplicationWindow):
         else:
             self.message_revealer.props.reveal_child = False
 
+    def __notification_present(self):
+        if self.stack.get_visible_child_name() != "cambalache":
+            self.notification_dialog.present(self)
 
+    def _on_notification_activate(self, action, data):
+        self.__notification_present()
+
+    def __on_new_notification(self, center, notification):
+        self.__notification_present()
+        self.__update_action_notification()
