@@ -219,6 +219,7 @@ class CmbMerengueProcess(GObject.Object):
             # Encode to binary first, before calculating lenght
             payload = payload.encode()
             cmd["payload_length"] = len(payload)
+            logger.debug(f"write_command {command} {len(payload)}")
 
         if args is not None:
             cmd["args"] = args
@@ -233,11 +234,18 @@ class CmbMerengueProcess(GObject.Object):
     def __socket_write_command(self, cmd, payload=None):
         # Send command in one line as json
         output_stream = self.__connection.props.output_stream
-        output_stream.write(json.dumps(cmd).encode())
-        output_stream.write(b"\n")
+
+        def write_data(data):
+            total_bytes = len(data)
+            total_sent = 0
+            while total_sent < total_bytes:
+                total_sent += output_stream.write(data[total_sent:])
+
+        write_data(json.dumps(cmd).encode())
+        write_data(b"\n")
 
         if payload is not None:
-            output_stream.write(payload)
+            write_data(payload)
 
         # Flush
         output_stream.flush()
@@ -273,7 +281,6 @@ class CmbView(Gtk.Box):
         self.__project = None
         self.__ui_id = 0
         self.__theme = None
-        self.__dark = False
 
         self.menu = self.__create_context_menu()
 
@@ -308,11 +315,16 @@ class CmbView(Gtk.Box):
         if os.path.exists(dirname):
             shutil.rmtree(dirname)
 
+    def __set_dark_mode(self, dark):
+        valid, bg_color = self.get_style_context().lookup_color('theme_bg_color')
+        if valid:
+            self.compositor.props.bg_color = bg_color
+
+        return GLib.SOURCE_REMOVE
+
     def _set_dark_mode(self, dark):
-        self.__dark = dark
-        bg_color = Gdk.RGBA()
-        bg_color.parse("gray18" if dark else "white")
-        self.compositor.props.bg_color = bg_color
+        # This needs to be called in an idle because theme_bg_color has not changed at this point
+        GLib.idle_add(self.__set_dark_mode, dark)
 
     def __merengue_command(self, command, payload=None, args=None):
         if self.__merengue.merengue_started:
